﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GanttView;
using Telerik.Windows.Controls.Scheduling;

namespace GanttAutoSummary
{
	public class ViewModel : ViewModelBase
	{
		public ViewModel()
		{
			this.addCommand = new DelegateCommand(OnAddCommandExecuted);
			this.removeCommand = new DelegateCommand(OnRemoveExecuted);
			
			this.visibleRange = new DateRange(DateTime.Today, DateTime.Today.AddDays(22));
			this.tasks = new ObservableCollection<IGanttTask>();

			this.LoadData();
		}		

		private ObservableCollection<IGanttTask> tasks;
		public ObservableCollection<IGanttTask> Tasks
		{
			get
			{
				return tasks;
			}
			set
			{
				tasks = value;
				OnPropertyChanged(() => Tasks);
			}
		}

		private IDateRange visibleRange;
		public IDateRange VisibleRange
		{
			get { return visibleRange; }
			set
			{
				visibleRange = value;
				OnPropertyChanged(() => this.VisibleRange);
			}
		}

		private DelegateCommand removeCommand;
		public DelegateCommand RemoveCommand
		{
			get { return removeCommand; }
			set { removeCommand = value; }
		}

		private DelegateCommand addCommand;
		public DelegateCommand AddCommand
		{
			get { return addCommand; }
			set { addCommand = value; }
		}
		
		private void LoadData()
		{
			var task1 = new GanttTask
			{
				Start = DateTime.Today,
				End = DateTime.Today.AddDays(7),
				Title = "Drag & Drop Items"
			};
			var task2 = new GanttTask
			{
				Start = DateTime.Today.AddDays(7),
				End = DateTime.Today.AddDays(12),
				Title = "Filtering Timeruler Items"
			};

			var task3 = new GanttTask
			{
				Start = DateTime.Today.AddDays(12).AddHours(8),
				End = DateTime.Today.AddDays(14),
				Title = "Highlighting of special slots"
			};

			var task4 = new GanttTask
			{
				Start = DateTime.Today.AddDays(15).AddHours(8),
				End = DateTime.Today.AddDays(15).AddHours(17),
				Title = "Design of the Examples"
			};

			var task5 = new GanttTask
			{
				Title = "Auto Summary",
				Children = { task1, task2, task3, task4 }
			};

			var range = this.CalculateRangeFor(task5);		

			task2.Dependencies.Add(new Dependency { FromTask = task1 });
			task3.Dependencies.Add(new Dependency { FromTask = task2 });
			task4.Dependencies.Add(new Dependency { FromTask = task3 });

			this.tasks.Add(task5);

			this.RefreshSummary(range);
		}

		private DateRange CalculateRangeFor(IGanttTask summary)
		{
			var range = new DateRange { Start = DateTime.MaxValue, End = DateTime.MinValue };

			foreach (var child in summary.Children.OfType<IGanttTask>())
			{
				if (child.Start < range.Start)
				{
					range.Start = child.Start;
				}
				if (child.End > range.End)
				{
					range.End = child.End;
				}
			}

			return range;
		}

		private void OnAddCommandExecuted(object parameter)
		{
			var lastTask = ((IList<IGanttTask>)this.tasks.FirstOrDefault().Children).Last();

			var task = new GanttTask
			{
				Start = DateTime.Today.AddDays(16).AddHours(8),
				End = DateTime.Today.AddDays(17).AddHours(17),
				Title = "Design of the Examples"
			};

			task.Dependencies.Add(new Dependency { FromTask = lastTask });

			((IList<IGanttTask>)this.tasks.FirstOrDefault().Children).Add(task);

			var range = this.CalculateRangeFor(this.tasks.FirstOrDefault());

			this.RefreshSummary(range);

			OnPropertyChanged(() => Tasks);
		}

		private void OnRemoveExecuted(object parameter)
		{
			var count = this.tasks.FirstOrDefault().Children.OfType<IGanttTask>().Count();

			((IList<IGanttTask>)this.tasks.FirstOrDefault().Children).RemoveAt(count - 1);

			var range = this.CalculateRangeFor(this.tasks.FirstOrDefault());

			this.RefreshSummary(range);

			OnPropertyChanged(() => Tasks);
		}

		private void RefreshSummary(DateRange range) 
		{
			this.tasks.FirstOrDefault().Start = range.Start;
			this.tasks.FirstOrDefault().End = range.End;
		}
	}
}
